import merge from 'lodash.merge';
import path from 'path';
import { log } from '../util/log';
import { defaultOptions } from './defaults';
import type { Configuration, FileConfig } from './types';

/**
 * The list of possible filenames for the configuration file. Package.json is checked separately.
 */
export const possibleFilenames = [
  'declarator.js',
  'declarator.json',
  '.declarator.js',
  '.declarator.json',
  '.declaratorrc',
  '.declaratorrc.js',
  '.declaratorrc.json'
];

/**
 *
 * Attempts to read or create a configuration file for the specified directory.
 *
 * @param directory the directory to find the file
 * @returns the read config or a newly created one
 */
export async function readConfig(
  directory = process.cwd()
): Promise<Partial<FileConfig>> {
  for (const possibleFile of possibleFilenames) {
    const name = path.join(directory, possibleFile);
    try {
      return require(name);
    } catch {
      log.debug`${name} could not be found.`;
      // no empty block
      continue;
    }
  }

  // Try finding it at package.json
  const packagePath = path.join(directory, 'package.json');
  try {
    const packageJson = await require(packagePath);
    const declarator = packageJson['declarator'];

    if (!declarator) {
      log.debug`${packagePath} does not contain a declarator section.`;
      // Skips to catch block
      throw null;
    }

    return declarator;
  } catch (err) {
    // no config found.
    log.info`No config file could be found. Tried (${possibleFilenames})`;

    return defaultOptions;
  }
}

/**
 * Parses the read config and merge any empty property with the defaults
 *
 * @param config the partial config that was read
 * @returns the complete config merged with defaults
 */
export function parseConfig(config: Partial<FileConfig>): Configuration {
  const read = typeof config === 'function' ? config() : config;
  return merge(defaultOptions, read);
}
